@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2012-2018, VU University Amsterdam
+    Copyright (c)  2012-2025, VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -38,6 +39,7 @@
 #ifdef __WINDOWS__
 #define LIBARCHIVE_STATIC 1
 #endif
+#include <stdbool.h>
 #include <archive.h>
 #include <archive_entry.h>
 #include <assert.h>
@@ -45,59 +47,9 @@
 #include <errno.h>
 #include <ctype.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#else					/* assume complete recent libarchive */
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_BZIP2 1
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_COMPRESS 1
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_GZIP 1
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_LZMA 1
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_NONE 1
-#define HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_XZ 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_AR 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_CPIO 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_EMPTY 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_ISO9660 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_MTREE 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_RAW 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_TAR 1
-#define HAVE_ARCHIVE_READ_SUPPORT_FORMAT_ZIP 1
-#define HAVE_ARCHIVE_READ_FREE 1
-#endif
-
-#ifndef HAVE_ARCHIVE_READ_FREE
-#define archive_read_free(ar) archive_read_finish(ar)
-#define archive_write_free(ar) archive_write_finish(ar)
-#endif
-
 #if ARCHIVE_VERSION_NUMBER < 3000000
-#define archive_read_support_filter_all archive_read_support_compression_all
-
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_BZIP2
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_BZIP2 1
-#define archive_read_support_filter_bzip2 archive_read_support_compression_bzip2
+#error "Requires libarchive 3.0.0 or later"
 #endif
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_COMPRESS
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_COMPRESS
-#define archive_read_support_filter_compress archive_read_support_compression_compress
-#endif
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_GZIP
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_GZIP
-#define archive_read_support_filter_gzip archive_read_support_compression_gzip
-#endif
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_LZMA
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_LZMA
-#define archive_read_support_filter_lzma archive_read_support_compression_lzma
-#endif
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_NONE
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_NONE
-#define archive_read_support_filter_none archive_read_support_compression_none
-#endif
-#ifdef HAVE_ARCHIVE_READ_SUPPORT_COMPRESSION_XZ
-#define HAVE_ARCHIVE_READ_SUPPORT_FILTER_XZ
-#define archive_read_support_filter_xz archive_read_support_compression_xz
-#endif
-#endif /*ARCHIVE_VERSION_NUMBER < 3000000*/
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -109,28 +61,130 @@ the archive, the current header and some state information.
 
 #define ARCHIVE_MAGIC 348184378
 
+/* The normal way of calling the predicates (similar for both read and
+   write).  [`ar` is the archive_wrapper blob]
+
+   For reading (ar->entry is allocated by archive_read_next_header):
+     archive_open_stream(Stream, read, Archive, Options),  % AR_VIRGIN -> AR_OPENED_ARCHIVE
+     repeat:
+       archive_next_header(Stream, Name),                  % AR_OPENED_ARCHIVE -> AR_NEW_ENTRY
+       archive_header_prop_(Archive, Property),
+       archive_open_entry(Archive, EntryStream),           % AR_NEW_ENTRY -> AR_OPENED_ENTRY
+       read(EntryStream, ...), get_code(EntryStream,Code), etc.
+       close(EntryStream),           % ar_entry_close_cb() % AR_OPENED_ENTRY _> AR_OPENED_ARCHIVE
+     archive_close(Archive),                               % ... -> AR_VIRGIN
+     close(Stream)
+
+   For writing (ar->entry is allocated by archive_entry_new()):
+     archive_open_stream(Stream, write, Archive, Options), % AR_VIRGIN -> AR_OPENED_ARCHIVE
+     repeat:
+       archive_next_header(Stream, Name),                  % AR_OPENED_ARCHIVE -> AR_NEW_ENTRY
+       archive_set_header_property(Archive, Property),
+       archive_open_entry(Archive, EntryStream),           % AR_NEW_ENTRY -> AR_OPENED_ENTRY
+       write(EntryStream, ...), format(EntryStream, ...), etc.
+       close(EntryStream),           % ar_entry_close_cb() % AR_OPENED_ENTRY _> AR_OPENED_ARCHIVE
+     archive_close(Archive),                               % ... -> AR_VIRGIN
+     close(Stream)
+
+   When the `ar` blob is created, it stores the Stream in ar->data by
+   calling PL_get_stream(). This increases the reference count for
+   the stream; it's decremented by PL_release_stream(ar->data).
+
+   When an EntryStream is created (by archive_open_entry/2) and a
+   write is done on the stream, the following calls and callbacks
+   happen for write (similar for read):
+      prolog:write(EntryStream)
+      archive4pl:ar_entry_write_cb()
+      libarchive:archive_write_data()
+      archive4pl:libarchive_write_cb()
+      Sfwrite(ar->data)
+
+   EntryStream needs `ar`, so archive_open_entry/2 ends up calling
+   PL_register_atom(ar->symbol) to ensure it doesn't get gc-ed.
+
+   The close for EntryStream - ar_entry_close_cb() - calls
+   archive_write_finish_entry() [for write]. It also calls
+   PL_unregister_atom(ar->symbol).
+
+   [The documentation for archive_write_finish_entry() says that it is
+   implicitly called by archive_write_header() or
+   archive_write_close(). However, it's simpler to explicitly call it
+   when closing the EntryStream.]
+
+   archive_close(Archive) calls archive_{read,write}_free(), which
+   calls archive_{read,write}_close(), which calls
+   libarchive_close_cb().
+
+   When the `ar` blob is released, it needs to free the
+   archive_entry struct, close the entry stream (EntryStream),
+   and close/release the archive Stream (ar->data).
+ */
 typedef enum ar_status
 { AR_VIRGIN = 0,
-  AR_OPENED,
+  AR_OPENED_ARCHIVE,
   AR_NEW_ENTRY,
   AR_OPENED_ENTRY,
-  AR_CLOSED_ENTRY
+  AR_CLOSED_ENTRY,
+  AR_ERROR
 } ar_status;
 
 typedef struct archive_wrapper
 { atom_t		symbol;		/* Associated symbol */
   IOSTREAM *		data;		/* Underlying stream */
-  unsigned int		type;		/* Type of format supported */
+  unsigned int		type;		/* Types of formats/filters supported */
   int			magic;		/* magic code */
   ar_status		status;		/* Current status */
   int			close_parent;	/* Close the parent handle */
   int			closed_archive;	/* Archive was closed with open entry */
   struct archive *	archive;	/* Actual archive handle */
   struct archive_entry *entry;		/* Current entry */
-  int                   how;            /* r/w mode */
+  int			how;		/* r/w mode ('r' or 'w') */
+  int			agc;		/* subject to AGC */
 } archive_wrapper;
 
-static void free_archive(archive_wrapper *ar);
+/* Convenience function - sets ar->status to AR_ERROR and returns false.
+   It is intended to be wrapped around PL_xxx_error() calls.  If
+   ar->status is set to AR_ERROR, that all further use of ar will
+   throw an error - this may be overkill for some errors. */
+
+static bool
+ar_set_status_error(archive_wrapper *ar, bool rc)
+{ if ( ar )
+    ar->status = AR_ERROR;
+  return false;
+}
+
+static archive_wrapper archive_wrapper_init_value =
+{
+  0,			/* symbol		*/
+  NULL,			/* data			*/
+  AR_VIRGIN,		/* type			*/
+  ARCHIVE_MAGIC,	/* magic		*/
+  AR_VIRGIN,		/* status		*/
+  FALSE,		/* close_parent		*/
+  FALSE,		/* closed_archive	*/
+  NULL,			/* archive		*/
+  NULL,			/* entry		*/
+  ' ',			/* how			*/
+  FALSE			/* agc			*/
+};
+
+#if 0
+/* For debugging: */
+static const char*
+ar_status_str(ar_status status)
+{ switch ( status )
+  { case AR_VIRGIN:         return "AR_VIRGIN";
+    case AR_OPENED_ARCHIVE: return "AR_OPENED";
+    case AR_NEW_ENTRY:      return "AR_NEW_ENTRY";
+    case AR_OPENED_ENTRY:   return "AR_OPENED_ENTRY";
+    case AR_CLOSED_ENTRY:   return "AR_CLOSED_ENTRY";
+    case AR_ERROR:          return "AR_ERROR";
+    default:                return "AR_???";
+  }
+}
+#endif
+
 
 
 		 /*******************************
@@ -208,42 +262,45 @@ int PL_existence_error3(const char* type, const char* object, term_t in)
 		 *******************************/
 
 static int archive_free_handle(archive_wrapper *ar)
-{ if ( ar->how == 'r' )
-    return archive_read_free(ar->archive);
+{ int rc;
+  /* It's safe to pass NULL to archive_read_free(), archive_write_free() */
+  if ( ar->how == 'r' )
+    rc = archive_read_free(ar->archive);
   else
-    return archive_write_free(ar->archive);
+    rc = archive_write_free(ar->archive);
+  ar->archive = NULL;
+  return rc;
 }
 
 static void
-acquire_archive(atom_t symbol)
+ar_w_acquire_cb(atom_t symbol)
 { archive_wrapper *ar = PL_blob_data(symbol, NULL, NULL);
   ar->symbol = symbol;
 }
 
-
+/* Callback from atom gc */
 static int
-release_archive(atom_t symbol)
+ar_w_release_cb(atom_t symbol)
 { archive_wrapper *ar = PL_blob_data(symbol, NULL, NULL);
-  struct archive *a;
 
-  assert(ar->status != AR_OPENED_ENTRY);
+  assert(ar->magic == ARCHIVE_MAGIC);
+  ar->agc = TRUE;
 
-  if ( (a=ar->archive) )
-  { ar->archive = NULL;
-    archive_free_handle(ar);
+  /* TODO: The following assert isn't always true */
+  /* assert(ar->status != AR_OPENED_ENTRY); */
 
-  }
-
-  free_archive(ar);
-  PL_free(ar);
+  /* Only writeable archives have ar->entry allocated by archive_entry_new() */
+  if (ar->how == 'w')
+    archive_entry_free(ar->entry); /* Safe even if !ar->entry */
+  archive_free_handle(ar);
 
   return TRUE;
 }
 
 static int
-compare_archives(atom_t a, atom_t b)
-{ archive_wrapper *ara = PL_blob_data(a, NULL, NULL);
-  archive_wrapper *arb = PL_blob_data(b, NULL, NULL);
+ar_w_compare_cb(atom_t a, atom_t b)
+{ const archive_wrapper *ara = PL_blob_data(a, NULL, NULL);
+  const archive_wrapper *arb = PL_blob_data(b, NULL, NULL);
 
   return ( ara > arb ?  1 :
 	   ara < arb ? -1 : 0
@@ -251,8 +308,8 @@ compare_archives(atom_t a, atom_t b)
 }
 
 static int
-write_archive(IOSTREAM *s, atom_t symbol, int flags)
-{ archive_wrapper *ar = PL_blob_data(symbol, NULL, NULL);
+ar_w_write_cb(IOSTREAM *s, atom_t symbol, int flags)
+{ const archive_wrapper *ar = PL_blob_data(symbol, NULL, NULL);
 
   Sfprintf(s, "<archive>(%p)", ar);
 
@@ -261,16 +318,16 @@ write_archive(IOSTREAM *s, atom_t symbol, int flags)
 
 static PL_blob_t archive_blob =
 { PL_BLOB_MAGIC,
-  PL_BLOB_NOCOPY,
+  0,
   "archive",
-  release_archive,
-  compare_archives,
-  write_archive,
-  acquire_archive
+  ar_w_release_cb,
+  ar_w_compare_cb,
+  ar_w_write_cb,
+  ar_w_acquire_cb
 };
 
 
-static int
+static bool
 get_archive(term_t t, archive_wrapper **arp)
 { PL_blob_t *type;
   void *data;
@@ -283,14 +340,13 @@ get_archive(term_t t, archive_wrapper **arp)
     if ( ar->symbol )
     { *arp = ar;
 
-      return TRUE;
+      return true;
     }
 
-    PL_permission_error("access", "closed_archive", t);
-    return FALSE;
+    return ar_set_status_error(ar, PL_permission_error("access", "closed_archive", t));
   }
 
-  return PL_type_error("archive", t);
+  return PL_type_error("archive", t),false;
 }
 
 
@@ -299,17 +355,18 @@ get_archive(term_t t, archive_wrapper **arp)
 		 *	      CALLBACKS		*
 		 *******************************/
 
+/* Callback from archive_XXX_close() or archive_XXX_free() */
 static int
-ar_open(struct archive *a, void *cdata)
+libarchive_open_cb(struct archive *a, void *cdata)
 { return ARCHIVE_OK;
 }
 
 static int
-ar_close(struct archive *a, void *cdata)
+libarchive_close_cb(struct archive *a, void *cdata)
 { archive_wrapper *ar = cdata;
   PL_release_stream(ar->data);
   if ( ar->close_parent && ar->archive )
-  { if ( Sclose(ar->data) != 0 )
+  { if ( Sgcclose(ar->data, ar->agc ? SIO_CLOSE_FORCE : 0) != 0 )
     { archive_set_error(ar->archive, errno, "Close failed");
       ar->data = NULL;
       return ARCHIVE_FATAL;
@@ -321,8 +378,16 @@ ar_close(struct archive *a, void *cdata)
 }
 
 static ssize_t
-ar_read(struct archive *a, void *cdata, const void **buffer)
-{ archive_wrapper *ar = cdata;
+libarchive_read_cb(struct archive *a, void *cdata, const void **buffer)
+{ const archive_wrapper *ar = cdata;
+  /* In the folowing code, Sfeof() call S__fillbuff() if the buffer is empty.
+     TODO: Why is the code written this way instead of using Sfread()?
+           One reason could be that apparently libarchive doesn't
+           provide the buffer to dump the data in so need to maintain
+           such a buffer ourselves and copy the data or use the
+           stream's buffer.  Looks a little hacky.  On the other hand,
+           there is little wrong with it and if something changes the
+           tests will tell us. */
   if ( Sfeof(ar->data) )
   { if ( Sferror(ar->data) )
       return -1;
@@ -339,29 +404,25 @@ ar_read(struct archive *a, void *cdata, const void **buffer)
 }
 
 static ssize_t
-ar_write(struct archive *a, void *cdata, const void *buffer, size_t n)
-{ archive_wrapper *ar = cdata;
+libarchive_write_cb(struct archive *a, void *cdata, const void *buffer, size_t n)
+{ const archive_wrapper *ar = cdata;
   return Sfwrite(buffer, 1, n, ar->data);
 }
 
-#ifndef __LA_INT64_T
-#define __LA_INT64_T off_t
-#endif
-
-static __LA_INT64_T
-ar_skip(struct archive *a, void *cdata, __LA_INT64_T request)
+static int64_t
+libarchive_skip_cb(struct archive *a, void *cdata, int64_t request)
 { archive_wrapper *ar = cdata;
 
   if ( Sseek64(ar->data, request, SIO_SEEK_CUR) == 0 )
     return request;
-  Sclearerr(ar->data);
+  Sclearerr(ar->data);			/* TODO: why is this? */
 
   return 0;				/* cannot skip; library will read */
 }
 
-#ifdef HAVE_ARCHIVE_READ_OPEN1
-static __LA_INT64_T
-ar_seek(struct archive *a, void *cdata, __LA_INT64_T request, int whence)
+#ifdef HAVE_ARCHIVE_READ_SET_SEEK_CALLBACK
+static int64_t
+libarchive_seek_cb(struct archive *a, void *cdata, int64_t request, int whence)
 { archive_wrapper *ar = cdata;
   int s_whence;
 
@@ -375,7 +436,7 @@ ar_seek(struct archive *a, void *cdata, __LA_INT64_T request, int whence)
   if ( Sseek64(ar->data, request, s_whence) == 0 ) {
     return Stell64(ar->data);
   }
-  Sclearerr(ar->data);				/* JW: why is this? */
+  Sclearerr(ar->data);				/* JW: TODO: why is this? */
 
   return ARCHIVE_FATAL;
 }
@@ -386,37 +447,38 @@ ar_seek(struct archive *a, void *cdata, __LA_INT64_T request, int whence)
 		 *	      PROLOG		*
 		 *******************************/
 
-static void
-free_archive(archive_wrapper *ar)
-{
-}
-
 static int
-archive_error(archive_wrapper *ar)
+archive_error(const archive_wrapper *ar, int rc)
 { int eno = archive_errno(ar->archive);
+  const char *s = archive_error_string(ar->archive);
+  term_t ex;
 
-  if ( eno != 0 )
-  { const char *s = archive_error_string(ar->archive);
-    term_t ex = PL_new_term_ref();
-
-    if ( PL_unify_term(ex,
-		       PL_FUNCTOR, FUNCTOR_error2,
-		         PL_FUNCTOR, FUNCTOR_archive_error2,
-		           PL_INT, errno,
-		           PL_CHARS, s,
-		         PL_VARIABLE) )
-      return PL_raise_exception(ex);
-
-    return FALSE;
+  if ( eno == 0 )
+    eno = rc;
+  if ( !s )
+  { switch( rc )
+    { case ARCHIVE_EOF:    s = "eof";    break;
+      case ARCHIVE_OK:     s = "ok";     break;
+      case ARCHIVE_RETRY:  s = "retry";  break;
+      case ARCHIVE_WARN:   s = "warn";   break;
+      case ARCHIVE_FAILED: s = "failed"; break;
+      case ARCHIVE_FATAL:  s = "fatal";  break;
+      default:		   s = "unknown";
+    }
   }
 
-  if ( PL_exception(0) )
-    return FALSE;
+  if ( ( (ex = PL_new_term_ref()) &&
+	 PL_unify_term(ex,
+		       PL_FUNCTOR, FUNCTOR_error2,
+			 PL_FUNCTOR, FUNCTOR_archive_error2,
+			   PL_INT, eno,
+			   PL_CHARS, s,
+			 PL_VARIABLE) ) )
+    return PL_raise_exception(ex);
 
-  return TRUE;
+  return FALSE;
 }
 
-
 #define	FILTER_ALL	  0x0000ffff
 #ifdef HAVE_ARCHIVE_READ_SUPPORT_FILTER_BZIP2
 #define	FILTER_BZIP2	  0x00000001
@@ -501,7 +563,12 @@ archive_error(archive_wrapper *ar)
 #endif
 
 #define FORMAT_MASK	  0xffff0000
+#if defined(FORMAT_RAW) && defined(FORMAT_MTREE)
 #define FORMAT_ALL	  (FORMAT_MASK&~(FORMAT_RAW|FORMAT_MTREE))
+#else
+/* Compile without these.  Make sure the constants are as above */
+#define FORMAT_ALL	  (FORMAT_MASK&~(0x04000000|0x01000000))
+#endif
 
 static void
 enable_type(archive_wrapper *ar, int type,
@@ -514,42 +581,45 @@ enable_type(archive_wrapper *ar, int type,
 
 static foreign_t
 archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
-{ IOSTREAM *datas;
-  archive_wrapper *ar;
+{ archive_wrapper *ar;
   term_t tail = PL_copy_term_ref(options);
   term_t head = PL_new_term_ref();
   term_t arg  = PL_new_term_ref();
-  atom_t mname;
-  char how = 'r';
-  int flags = 0;
+  int rc = ARCHIVE_OK;				/* silence compiler */
 
-  if ( PL_get_atom(mode, &mname) )
-  { if ( mname == ATOM_write )
-    { how = 'w';
-      flags = SIO_OUTPUT;
-    }
-    else if ( mname == ATOM_read )
-    { how = 'r';
-      flags = SIO_INPUT;
-    }
-    else
-    { PL_domain_error("io_mode", mode);
-    }
-  } else
-  { PL_type_error("atom", mode);
+  /* If you make any changes here, also change archive_close() */
+
+  { archive_wrapper ar_local;
+    atom_t a;
+    ar_local = archive_wrapper_init_value;
+
+    if ( !PL_unify_blob(handle, &ar_local, sizeof ar_local, &archive_blob) ||
+         !PL_get_atom_ex(handle, &a) )
+      return FALSE;
+    ar = PL_blob_data(a, NULL, NULL);
   }
 
-  if ( !PL_get_stream(data, &datas, flags) )
-    return FALSE;
 
-  ar = PL_malloc(sizeof(*ar));
-  memset(ar, 0, sizeof(*ar));
-  ar->data = datas;
-  ar->how = how;
-  ar->magic = ARCHIVE_MAGIC;
+  { atom_t mname;
+    int flags;
 
-  if ( !PL_unify_blob(handle, ar, sizeof(*ar), &archive_blob) )
-    return FALSE;
+    if ( PL_get_atom_ex(mode, &mname) )
+    { if ( mname == ATOM_write )
+      { ar->how = 'w';
+        flags = SIO_OUTPUT;
+      } else if ( mname == ATOM_read )
+      { ar->how = 'r';
+        flags = SIO_INPUT;
+      } else
+      { return ar_set_status_error(ar, PL_domain_error("io_mode", mode));
+      }
+    } else
+    { return FALSE;
+    }
+
+    if ( !PL_get_stream(data, &ar->data, flags) )
+      return FALSE;
+  }
 
   while( PL_get_list_ex(tail, head, tail) )
   { atom_t name;
@@ -557,17 +627,17 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
 
     if ( !PL_get_name_arity(head, &name, &arity) ||
 	 !PL_get_arg(1, head, arg) )
-      return PL_type_error("option", head);
+      return ar_set_status_error(ar, PL_type_error("option", head));
     if ( name == ATOM_compression || name == ATOM_filter )
     { atom_t c;
 
       if ( !PL_get_atom_ex(arg, &c) )
 	return FALSE;
-      if ( how == 'w' && ((ar->type & FILTER_MASK) != 0) )
-         return PL_permission_error("set", "filter", arg);
+      if ( ar->how == 'w' && ((ar->type & FILTER_MASK) != 0) )
+        return ar_set_status_error(ar, PL_permission_error("set", "filter", arg));
       if ( c == ATOM_all )
-      { if (how == 'w')
-          return PL_domain_error("write_filter", arg);
+      { if (ar->how == 'w')
+          return ar_set_status_error(ar, PL_domain_error("write_filter", arg));
 	ar->type |= FILTER_ALL;
       }
 #ifdef FILTER_BZIP2
@@ -619,17 +689,17 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
 	ar->type |= FILTER_XZ;
 #endif
       else
-	return PL_domain_error("filter", arg);
+	return ar_set_status_error(ar, PL_domain_error("filter", arg));
     } else if ( name == ATOM_format )
     { atom_t f;
 
       if ( !PL_get_atom_ex(arg, &f) )
 	return FALSE;
-      if ( how == 'w' && (( ar->type & FORMAT_MASK ) != 0 ) )
-          return PL_permission_error("set", "format", arg);
+      if ( ar->how == 'w' && (( ar->type & FORMAT_MASK ) != 0 ) )
+        return ar_set_status_error(ar, PL_permission_error("set", "format", arg));
       if ( f == ATOM_all )
-      { if ( how == 'w' )
-          return PL_domain_error("write_format", arg);
+      { if ( ar->how == 'w' )
+          return ar_set_status_error(ar, PL_domain_error("write_format", arg));
 	ar->type |= FORMAT_ALL;
       }
 #ifdef FORMAT_7ZIP
@@ -689,7 +759,7 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
 	ar->type |= FORMAT_ZIP;
 #endif
       else
-	return PL_domain_error("format", arg);
+	return ar_set_status_error(ar, PL_domain_error("format", arg));
     } else if ( name == ATOM_close_parent )
     { if ( !PL_get_bool_ex(arg, &ar->close_parent) )
 	return FALSE;
@@ -698,15 +768,15 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
   if ( !PL_get_nil_ex(tail) )
     return FALSE;
 
-  if ( how == 'r' )
+  if ( ar->how == 'r' )
   { if ( !(ar->type & FILTER_ALL) )
       ar->type |= FILTER_ALL;
     if ( !(ar->type & FORMAT_MASK) )
       ar->type |= FORMAT_ALL;
   }
-  if ( how == 'r' )
+  if ( ar->how == 'r' )
   { if ( !(ar->archive = archive_read_new()) )
-      return PL_resource_error("memory");
+      return ar_set_status_error(ar, PL_resource_error("memory"));
 
      if ( (ar->type & FILTER_ALL) == FILTER_ALL )
      { archive_read_support_filter_all(ar->archive);
@@ -800,28 +870,25 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
        enable_type(ar, FORMAT_ZIP,     archive_read_support_format_zip);
 #endif
      }
-#ifdef HAVE_ARCHIVE_READ_OPEN1
      archive_read_set_callback_data(ar->archive, ar);
-     archive_read_set_open_callback(ar->archive, ar_open);
-     archive_read_set_read_callback(ar->archive, ar_read);
-     archive_read_set_skip_callback(ar->archive, ar_skip);
-     archive_read_set_seek_callback(ar->archive, ar_seek);
-     archive_read_set_close_callback(ar->archive, ar_close);
-
-     if ( archive_read_open1(ar->archive) == ARCHIVE_OK )
-     { ar->status = AR_OPENED;
-       return TRUE;
-     }
-#else
-     if ( archive_read_open2(ar->archive, ar,
-                             ar_open, ar_read, ar_skip, ar_close) == ARCHIVE_OK )
-     { ar->status = AR_OPENED;
-       return TRUE;
-     }
+     archive_read_set_open_callback(ar->archive, libarchive_open_cb);
+     archive_read_set_read_callback(ar->archive, libarchive_read_cb);
+     archive_read_set_skip_callback(ar->archive, libarchive_skip_cb);
+#ifdef HAVE_ARCHIVE_READ_SET_SEEK_CALLBACK
+     archive_read_set_seek_callback(ar->archive, libarchive_seek_cb);
 #endif
-  } else if ( how == 'w' )
+     archive_read_set_close_callback(ar->archive, libarchive_close_cb);
+
+     if ( (rc=archive_read_open1(ar->archive)) == ARCHIVE_OK )
+     { ar->status = AR_OPENED_ARCHIVE;
+       return TRUE;
+     }
+  } else if ( ar->how == 'w' )
   { if ( !(ar->archive = archive_write_new()) )
-      return PL_resource_error("memory");
+      return ar_set_status_error(ar, PL_resource_error("memory"));
+     /* Prevent libarchive from padding the last block to 10240 bytes. Some decompressors,
+       notably Oracle's jar decompressor, fail when presented with this */
+     archive_write_set_bytes_in_last_block(ar->archive, 1);
      if (0) {}
 #ifdef FORMAT_7ZIP
      else if ( ar->type & FORMAT_7ZIP )    archive_write_set_format_7zip(ar->archive);
@@ -842,7 +909,7 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
     else if ( ar->type & FORMAT_ZIP )      archive_write_set_format_zip(ar->archive);
 #endif
     else
-    { return PL_existence_error3("option", "format", options);
+    { return ar_set_status_error(ar, PL_existence_error3("option", "format", options));
     }
 
 
@@ -875,29 +942,19 @@ archive_open_stream(term_t data, term_t mode, term_t handle, term_t options)
     else                                   archive_write_add_filter_none(ar->archive);
 #else
     else
-    { return PL_existence_error3("option", "filter", options);
+    { return ar_set_status_error(ar, PL_existence_error3("option", "filter", options));
     }
 #endif
-#ifdef HAVE_ARCHIVE_WRITE_OPEN1
-    archive_write_set_callback_data(ar->archive, ar);
-    archive_write_set_open_callback(ar->archive, ar_open);
-    archive_write_set_write_callback(ar->archive, ar_write);
-    archive_write_set_close_callback(ar->archive, ar_close);
-
-    if ( archive_write_open1(ar->archive) == ARCHIVE_OK )
-    { ar->status = AR_OPENED;
+    if ( (rc=archive_write_open(ar->archive, ar,
+				libarchive_open_cb, libarchive_write_cb, libarchive_close_cb)) == ARCHIVE_OK )
+    { ar->status = AR_OPENED_ARCHIVE;
       return TRUE;
     }
-#else
-    if ( archive_write_open(ar->archive, ar,
-                             ar_open, ar_write, ar_close) == ARCHIVE_OK )
-    { ar->status = AR_OPENED;
-      return TRUE;
-    }
-#endif
+  } else {
+    assert(0);
   }
 
-  return archive_error(ar);
+  return archive_error(ar, rc);
 }
 
 
@@ -911,7 +968,6 @@ archive_property(term_t archive, term_t prop, term_t value)
        !PL_get_atom_ex(prop, &pn) )
     return FALSE;
 
-#ifdef HAVE_ARCHIVE_FILTER_COUNT
   if ( pn == ATOM_filter )
   { int i, fcount = archive_filter_count(ar->archive);
     term_t tail = PL_copy_term_ref(value);
@@ -929,7 +985,6 @@ archive_property(term_t archive, term_t prop, term_t value)
     }
     return PL_unify_nil(tail);
   }
-#endif
 
   return FALSE;
 }
@@ -945,12 +1000,15 @@ archive_next_header(term_t archive, term_t name)
   if ( ar->how == 'w' )
   { char* pathname = NULL;
     if ( ar->status == AR_OPENED_ENTRY )
-       return PL_permission_error("next_header", "archive", archive);
+      return ar_set_status_error(ar, PL_permission_error("next_header", "archive", archive));
     if ( !PL_get_atom_chars(name, &pathname) )
-       return PL_type_error("atom", name);
-    ar->entry = archive_entry_new();
-    if ( ar->entry == NULL )
-       return PL_resource_error("memory");
+      return ar_set_status_error(ar, PL_type_error("atom", name));
+    if ( ar->entry )
+      archive_entry_clear(ar->entry);
+    else
+      ar->entry = archive_entry_new();
+    if ( !ar->entry )
+      return ar_set_status_error(ar, PL_resource_error("memory"));
     archive_entry_set_pathname(ar->entry, pathname);
     /* libarchive-3.1.2 does not tolerate an empty size with zip. Later versions may though - it is fixed in git as of Dec 2013.
      *    For now, set the other entries to a sensible default
@@ -962,9 +1020,11 @@ archive_next_header(term_t archive, term_t name)
     return TRUE;
   }
   if ( ar->status == AR_NEW_ENTRY )
-    archive_read_data_skip(ar->archive);
-  if ( ar->status == AR_OPENED_ENTRY )
-    return PL_permission_error("next_header", "archive", archive);
+  { if ( (rc=archive_read_data_skip(ar->archive)) != ARCHIVE_OK )
+      return archive_error(ar, rc);
+  } else if ( ar->status == AR_OPENED_ENTRY )
+  { return ar_set_status_error(ar, PL_permission_error("next_header", "archive", archive));
+  }
 
   while ( (rc=archive_read_next_header(ar->archive, &ar->entry)) == ARCHIVE_OK )
   { if ( PL_unify_wchars(name, PL_ATOM, -1,
@@ -979,7 +1039,7 @@ archive_next_header(term_t archive, term_t name)
   if ( rc == ARCHIVE_EOF )
     return FALSE;			/* simply at the end */
 
-  return archive_error(ar);
+  return archive_error(ar, rc);
 }
 
 
@@ -1003,7 +1063,7 @@ archive_close(term_t archive)
     return TRUE;
   }
 
-  return archive_error(ar);
+  return archive_error(ar, rc);
 }
 
 		 /*******************************
@@ -1011,7 +1071,7 @@ archive_close(term_t archive)
 		 *******************************/
 
 static foreign_t
-archive_header_prop(term_t archive, term_t field)
+archive_header_prop_(term_t archive, term_t field)
 { archive_wrapper *ar;
   functor_t prop;
 
@@ -1019,9 +1079,9 @@ archive_header_prop(term_t archive, term_t field)
     return FALSE;
 
   if ( !PL_get_functor(field, &prop) )
-    return PL_type_error("compound", field);
+    return ar_set_status_error(ar, PL_type_error("compound", field));
   if ( ar->status != AR_NEW_ENTRY )
-    return PL_permission_error("access", "archive_entry", archive);
+    return ar_set_status_error(ar, PL_permission_error("access", "archive_entry", archive));
 
   if ( prop == FUNCTOR_filetype1 )
   { __LA_MODE_T type = archive_entry_filetype(ar->entry);
@@ -1087,7 +1147,7 @@ archive_header_prop(term_t archive, term_t field)
       _PL_get_arg(1, field, arg);
 
       for(o=lwr; *s && o < lwr+sizeof(lwr); )
-	*o++ = tolower(*s++);
+	*o++ = (char)tolower(*s++);
 
       *o = '\0';
 
@@ -1095,7 +1155,7 @@ archive_header_prop(term_t archive, term_t field)
     }
   }
 
-  return PL_domain_error("archive_header_property", field);
+  return ar_set_status_error(ar, PL_domain_error("archive_header_property", field));
 }
 
 static foreign_t
@@ -1107,11 +1167,11 @@ archive_set_header_property(term_t archive, term_t field)
     return FALSE;
 
   if ( !PL_get_functor(field, &prop) )
-    return PL_type_error("compound", field);
+    return ar_set_status_error(ar, PL_type_error("compound", field));
   if ( ar->status != AR_NEW_ENTRY )
-    return PL_permission_error("access", "archive_entry", archive);
+    return ar_set_status_error(ar, PL_permission_error("access", "archive_entry", archive));
   if ( ar->how != 'w' )
-    return PL_permission_error("write", "archive_entry", archive);
+    return ar_set_status_error(ar, PL_permission_error("write", "archive_entry", archive));
 
   if ( prop == FUNCTOR_filetype1 )
   { atom_t name;
@@ -1119,7 +1179,7 @@ archive_set_header_property(term_t archive, term_t field)
     __LA_MODE_T type;
     _PL_get_arg(1, field, arg);
     if ( !PL_get_atom(arg, &name) )
-       return PL_type_error("atom", arg);
+      return ar_set_status_error(ar, PL_type_error("atom", arg));
     if (name == ATOM_file)                  type = AE_IFREG;
     else if (name == ATOM_link)             type = AE_IFLNK;
     else if (name == ATOM_socket)           type = AE_IFSOCK;
@@ -1128,7 +1188,7 @@ archive_set_header_property(term_t archive, term_t field)
     else if (name == ATOM_directory)        type = AE_IFDIR;
     else if (name == ATOM_fifo)             type = AE_IFIFO;
     else
-       return PL_domain_error("filetype", arg);
+      return ar_set_status_error(ar, PL_domain_error("filetype", arg));
     archive_entry_set_filetype(ar->entry, type);
     PL_succeed;
   } else if (prop == FUNCTOR_mtime1)
@@ -1136,7 +1196,7 @@ archive_set_header_property(term_t archive, term_t field)
     term_t arg = PL_new_term_ref();
     _PL_get_arg(1, field, arg);
     if ( !PL_get_float(arg, &mtime) )
-       return PL_type_error("float", arg);
+      return ar_set_status_error(ar, PL_type_error("float", arg));
     archive_entry_set_mtime(ar->entry, (time_t)mtime, 0);
     PL_succeed;
   } else if (prop == FUNCTOR_size1)
@@ -1144,7 +1204,7 @@ archive_set_header_property(term_t archive, term_t field)
     term_t arg = PL_new_term_ref();
     _PL_get_arg(1, field, arg);
     if ( !PL_get_int64(arg, &size) )
-       return PL_type_error("size", arg);
+      return ar_set_status_error(ar, PL_type_error("size", arg));
     archive_entry_set_size(ar->entry, size);
     PL_succeed;
   } else if (prop == FUNCTOR_link_target1)
@@ -1154,13 +1214,13 @@ archive_set_header_property(term_t archive, term_t field)
     term_t arg = PL_new_term_ref();
     _PL_get_arg(1, field, arg);
     if ( !PL_get_atom(arg, &atom) )
-      return PL_type_error("atom", arg);
+      return ar_set_status_error(ar, PL_type_error("atom", arg));
     link = PL_atom_wchars(atom, &len);
     archive_entry_copy_symlink_w(ar->entry, link);
     archive_entry_set_filetype(ar->entry, AE_IFLNK);
     PL_succeed;
   } else
-    return PL_domain_error("archive_header_property", field);
+    return ar_set_status_error(ar, PL_domain_error("archive_header_property", field));
 }
 
 
@@ -1169,22 +1229,22 @@ archive_set_header_property(term_t archive, term_t field)
 		 *******************************/
 
 static ssize_t
-ar_read_entry(void *handle, char *buf, size_t size)
-{ archive_wrapper *ar = handle;
+ar_entry_read_cb(void *handle, char *buf, size_t size)
+{ const archive_wrapper *ar = handle;
 
   return archive_read_data(ar->archive, buf, size);
 }
 
 static ssize_t
-ar_write_entry(void *handle, char *buf, size_t size)
-{ archive_wrapper *ar = handle;
+ar_entry_write_cb(void *handle, char *buf, size_t size)
+{ const archive_wrapper *ar = handle;
 
   size_t written = archive_write_data(ar->archive, buf, size);
   /* In older version of libarchive (at least until 3.1.12), archive_write_data returns 0 for
      some formats if the file size is not set. It does not set archive_errno(), unfortunately.
      We turn this into an IO error here by returning -1
   */
-  if (written == 0)
+  if (written == 0 && size > 0)
   { errno = ENOSPC;
     return -1;
   }
@@ -1193,7 +1253,7 @@ ar_write_entry(void *handle, char *buf, size_t size)
 
 
 static int
-ar_close_entry(void *handle)
+ar_entry_close_cb(void *handle)
 { archive_wrapper *ar = handle;
   if ( ar->closed_archive )
   { if ( ar->archive )
@@ -1215,8 +1275,8 @@ ar_close_entry(void *handle)
 }
 
 static int
-ar_control_entry(void *handle, int op, void *data)
-{ archive_wrapper *ar = handle;
+ar_entry_control_cb(void *handle, int op, void *data)
+{ const archive_wrapper *ar = handle;
 
   (void)ar;
 
@@ -1234,38 +1294,40 @@ ar_control_entry(void *handle, int op, void *data)
 }
 
 static IOFUNCTIONS ar_entry_read_functions =
-{ ar_read_entry,
-  NULL,
+{ ar_entry_read_cb,
+  NULL,					/* write */
   NULL,					/* seek */
-  ar_close_entry,
-  ar_control_entry,			/* control */
+  ar_entry_close_cb,
+  ar_entry_control_cb,			/* control */
   NULL,					/* seek64 */
 };
 
 static IOFUNCTIONS ar_entry_write_functions =
-{ NULL,
-  ar_write_entry,
+{ NULL,					/* read */
+  ar_entry_write_cb,
   NULL,					/* seek */
-  ar_close_entry,
-  ar_control_entry,			/* control */
+  ar_entry_close_cb,
+  ar_entry_control_cb,			/* control */
   NULL,					/* seek64 */
 };
 
 static foreign_t
 archive_open_entry(term_t archive, term_t stream)
 { archive_wrapper *ar;
-  IOSTREAM *s;
+  IOSTREAM *entry_data;
+
+  /* If you make changes here, be sure to also change ar_entry_close() */
 
   if ( !get_archive(archive, &ar) )
     return FALSE;
   if ( ar->how == 'r' )
-  { if ( (s=Snew(ar, SIO_INPUT|SIO_RECORDPOS, &ar_entry_read_functions)) )
+  { if ( (entry_data=Snew(ar, SIO_INPUT|SIO_RECORDPOS, &ar_entry_read_functions)) )
     { ar->status = AR_OPENED_ENTRY;
-      if ( PL_unify_stream(stream, s) )
+      if ( PL_unify_stream(stream, entry_data) )
       { PL_register_atom(ar->symbol);	/* We may no longer reference the */
         return TRUE;			/* archive itself */
       }
-      Sclose(s);
+      Sclose(entry_data);
       return FALSE;
     }
   } else if ( ar->how == 'w' )
@@ -1273,17 +1335,18 @@ archive_open_entry(term_t archive, term_t stream)
     if ( ar->status == AR_NEW_ENTRY )
     { archive_write_header(ar->archive, ar->entry);
       archive_entry_free(ar->entry);
+      ar->entry = NULL;
     } else
-    { return PL_permission_error("access", "archive_entry", archive);
+    { return ar_set_status_error(ar, PL_permission_error("access", "archive_entry", archive));
     }
     /* Then we can make a handle for the data */
-    if ( (s=Snew(ar, SIO_OUTPUT|SIO_RECORDPOS, &ar_entry_write_functions)) )
+    if ( (entry_data=Snew(ar, SIO_OUTPUT|SIO_RECORDPOS, &ar_entry_write_functions)) )
     { ar->status = AR_OPENED_ENTRY;
-      if ( PL_unify_stream(stream, s) )
+      if ( PL_unify_stream(stream, entry_data) )
       { PL_register_atom(ar->symbol);	/* We may no longer reference the */
         return TRUE;			/* archive itself */
       }
-      Sclose(s);
+      Sclose(entry_data);
       return FALSE;
     }
   }
@@ -1358,7 +1421,7 @@ install_archive4pl(void)
   PL_register_foreign("archive_property",     3, archive_property,    0);
   PL_register_foreign("archive_close",        1, archive_close,       0);
   PL_register_foreign("archive_next_header",  2, archive_next_header, 0);
-  PL_register_foreign("archive_header_prop_", 2, archive_header_prop, 0);
+  PL_register_foreign("archive_header_prop_", 2, archive_header_prop_, 0);
   PL_register_foreign("archive_set_header_property", 2, archive_set_header_property, 0);
   PL_register_foreign("archive_open_entry",   2, archive_open_entry,  0);
 }
