@@ -92,6 +92,11 @@ test(hello, Out == "hello hello hello") :-
     % hello :- write('hello hello hello')
     with_output_to(string(Out), hello).
 
+test(helloex, error(unknown_error('an exception'))) :-
+    helloex.
+test(helloex, error(unknown_error(foo(bar)))) :-
+    helloex(foo(bar)).
+
 test(hello, Out == "Hello WORLD\nHello WORLD\nHello WORLD\nHello WORLD\nHello WORLD\n") :-
     hello("WORLD", Out).
 test(hello, error(representation_error(encoding))) :-
@@ -283,6 +288,7 @@ test(cappend, fail) :-
 test(cappend, fail) :-
     cappend([a,b,c], [d,e], [a,b,c,d,e|f]).
 
+%% TODO: add more tests of cpp_call/2
 test(cpp_call, Out == "abc\n") :-
     with_output_to(string(Out),
 		   cpp_call(writeln(abc), [normal])).
@@ -291,6 +297,13 @@ cpp_call(Goal, Flags) :-
     query_flags(Flags, CombinedFlag),
     cpp_call_(Goal, CombinedFlag, false).
 
+test(cpp_atom_codes, Out == [102,111,111]) :-
+    cpp_atom_codes(foo, Out).
+test(cpp_atom_codes, Out == [49,50,51]) :-
+    cpp_atom_codes(123, Out).
+test(cpp_atom_codes, error(type_error(atom,a(f)))) :-
+    cpp_atom_codes(a(f), _Out).
+
 test(square_roots_2, Result == [0.0, 1.0, 1.4142135623730951, 1.7320508075688772, 2.0]) :-
     square_roots(4, Result).
 
@@ -948,7 +961,7 @@ test(file_blob, Read == "% -*- mode: Prolog; coding: utf-8 -*-\n\n") :-
     print(File), nl,
     absolute_file_name(my_program_home('test_cpp.pl'), Abs, [access(read)]),
     expected_file_name_my_program_home('test_cpp.pl', AbsPathOS_atom),
-    assertion(AbsPathOS_atom == Abs),
+    assertion(prolog_to_os_filename(Abs, AbsPathOS_atom)),
     my_file_filename_atom(File, Filename),
     assertion(Filename == Abs),
     my_file_read(File, 39, Read),
@@ -958,11 +971,10 @@ test(file_blob, Read == "% -*- mode: Prolog; coding: utf-8 -*-\n\n") :-
 %     expected_file_name_my_program_home('non-existent-file', F0),
 %     atom_string(F0, F),
 %     my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath]).
-test(file_blob) :-
+test(file_blob, error((existence_error(my_file_blob_open,PlF)))) :-
     expected_file_name_my_program_home_string('non-existent-file', F),
-    catch(my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath]),
-          E,
-          assertion(E =@= error(existence_error(my_file_blob_open,F),_))).
+    prolog_to_os_filename(PlF, F),
+    my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath]).
 test(file_blob, error(existence_error(source_sink,my_program_home('non-existent-file')))) :-
     my_file_open(_File, my_program_home('non-existent-file'), 'r', [search,absolute,ospath,read]).
 test(file_blob, error(existence_error(source_sink,my_program_home('non-existent-file')))) :-
