@@ -122,7 +122,7 @@ const char *tex_error_strings[] =
 
 #define CMD_MODIFY	0x01		/* command allows for modifier */
 
-#define PRE_COMMENT	-1		/* put %\n before command */
+#define PRE_COMMENT	((char)-1)	/* put %\n before command */
 
 typedef struct _command *Command;
 typedef struct _environment *Environment;
@@ -1089,7 +1089,7 @@ env_verbatim(Environment e, Input fd, CallBack func, void *ctx)
   int el;
   token t;
 
-  sprintf(end, "\\end{%s}", e->environment->name);
+  snprintf(end, sizeof(end), "\\end{%s}", e->environment->name);
   el = (int)strlen(end);
   ms = buf+el;
 
@@ -2225,7 +2225,7 @@ build_list(Token t, void *context)
     case TOK_CMD:
     { Command g       = t->value.cmd;
       term_t alist    = PL_new_term_ref();
-      term_t modified = (g->flags & CMD_MODIFY ? ATOM_star : ATOM_minus);
+      atom_t modified = (g->flags & CMD_MODIFY ? ATOM_star : ATOM_minus);
       int rc;
 
       if ( g->command->flags & CMD_MODIFY )
@@ -2393,7 +2393,7 @@ pl_tex_tokens(term_t file, term_t tokens)
   if ( PL_get_chars(file, &fname, CVT_ALL) )
   { pl_context ctx;
 
-    ctx.list       = tokens;
+    ctx.list       = PL_copy_term_ref(tokens);
     ctx.head       = PL_new_term_ref();
     ctx.envnesting = 0;
     ctx.prev_type0 = TOK_EOF;
@@ -2461,7 +2461,7 @@ pl_tex_tell(term_t file)
 
 
 foreign_t
-pl_tex_told()
+pl_tex_told(void)
 { fflush(ppctx.fd);
   if ( ppctx.fd != stdout )
     fclose(ppctx.fd);
