@@ -41,6 +41,7 @@
 :- use_module(library(tabbed_window)).
 :- use_module(prompt).
 :- use_module(library(pce_util)).
+:- use_module(library(debug)).
 
 :- require([ between/3,
              atomic_list_concat/2,
@@ -858,7 +859,14 @@ editor_idle_event(E) :->
 
 start_idle_timer(E, Interval:[real]) :->
     "Reset the idle timer to timeout after the specified time"::
-    default(Interval, 2, Time),
+    (   Interval == @default
+    ->  get(E, mode, Mode),
+        (   Mode \== @nil
+        ->  get(Mode, idle_timeout, Time)
+        ;   Time = 2
+        )
+    ;   Time = Interval
+    ),
     send(@emacs_idle_timer, interval, Time),
     send(@emacs_idle_timer, status, once),
     send(@emacs_idle_timer, delete_hypers),
@@ -902,10 +910,11 @@ event(E, Ev:event) :->
         (   get(Frame, transients, Transients),
             Transients \== @nil,
             get(Transients, find, @arg1?modal == transient, _)
-        ->  format('We have a transient~n')
+        ->  debug(transient, 'We have a transient~n', [])
         ;   send(Frame, keyboard_focus, E?window)
         )
-    ;   send_super(E, event, Ev)
+    ;   send_super(E, event, Ev),
+        send(@emacs, editor_event, Ev)
     ).
 
 
@@ -958,6 +967,20 @@ drop(E, Obj:object) :->
     send(Mode, drop, Obj).
 
 
+selected_fragment(E, Fragment:fragment) :->
+    "User selected a fragment in the margin"::
+    send_super(E, selected_fragment, Fragment),
+    get(E, mode, Mode),
+    get(Mode?class, send_method, selected_fragment, _), % avoid delegation
+    send(Mode, selected_fragment, Fragment).
+
+hover_fragment_icon(E, Fragment:fragment*, Area:[area]) :->
+    "User hovers the fragment icon"::
+    send_super(E, hover_fragment_icon, Fragment, Area),
+    get(E, mode, Mode),
+    get(Mode?class, send_method, hover_fragment_icon, _), % avoid delegation
+    send(Mode, hover_fragment_icon, Fragment, Area).
+
 auto_fill(E, Caret:[int], Regex:[regex]) :->
     "Delegate to mode"::
     (   get(E, mode, Mode),
@@ -968,7 +991,7 @@ auto_fill(E, Caret:[int], Regex:[regex]) :->
 
 
 import_selection(E) :->
-    "Import the (primary) selection or the cut_buffer"::
+    "Import the (primary) selection"::
     get(E, display, Display),
     get(Display, selected_text, String),
     (   get(E, frame, Frame),
@@ -1156,6 +1179,9 @@ variable(m_x_history,     chain*,       both, "Current M-x command history").
 variable(m_x_index,       int*,         both, "M-p/M-n current index").
 variable(m_x_argn,        int*,         both, "M-p/M-n current argument").
 variable(keep_selection,  bool := @off, both, "Keep selection for this method").
+variable(idle_timeout,    num,		both, "Auto highlighting timeout").
+
+class_variable(idle_timeout, num, 2).
 
 delegate_to(editor).
 
@@ -1171,7 +1197,9 @@ initialise(M) :->
     send(M, send_super, initialise),
     send(M, slot, name, Name),
     send(M, slot, syntax, Name),              % converts to object
-    send(M, bindings).
+    send(M, load_user_extensions),
+    send(M, bindings),
+    send(M, obtain_class_variables).
 
 
 icon(_, I:image) :<-
@@ -1398,9 +1426,9 @@ arg_call(M, Selector:name, Arg:any) :->
                  *           SETUP              *
                  *******************************/
 
-setup_mode(M) :->
-    "Initialise mode (->load_user_extensions)"::
-    send(M, load_user_extensions).
+setup_mode(_M) :->
+    "Initialise mode (virtual)"::
+    true.
 
 
 load_user_extensions(M) :->
@@ -1600,6 +1628,15 @@ report_to(M, E:editor) :<-
     "Send reports to the <-editor"::
     get(M, editor, E).
 
+
+                /*******************************
+                *       LSP INTEGRATION        *
+                *******************************/
+
+lsp_client(_M, _Client) :<-
+    "LSP client associated to this mode (virtual)"::
+    fail.
+
 :- pce_end_class.
 
 
