@@ -711,15 +711,9 @@ typedef struct
   Name		kind;			/* {pixmap,bitmap} */ \
   SourceSink	file;			/* Resolved file */ \
   Name		access;			/* {read,both} */ \
-  Colour	background;		/* Background-colour (pixmap) */ \
-  Colour	foreground;		/* Foreground-colour (pixmap) */ \
-  Int		depth;			/* Bits/pixel */ \
   Size		size;			/* Size of the image */ \
-  Real		scale;			/* Scale image at load time */ \
-  DisplayObj	display;		/* Display of read-write's */ \
   BitmapObj	bitmap;			/* Bitmap for read-write's */ \
   Point		hot_spot;		/* Indication of hot-spot */ \
-  Image		mask;			/* Masking image */ \
   WsRef		ws_ref;			/* Window system reference */ \
   builtin_image_data *bits;		/* Data for built-in images */
 
@@ -734,7 +728,6 @@ End;
 NewClass(bitmapobj)
   ABSTRACT_GRAPHICAL
   Image		image;			/* Image of the bitmap */
-  BoolObj	transparent;		/* Just stipple pattern */
 End;
 
 
@@ -769,9 +762,11 @@ NewClass(fontobj)
   Name		style;			/* Style */
   Any		weight;			/* Weight */
   Num		points;			/* Point size */
-  Num		ex;			/* Width of an x in this font */
-  Num		avg_char_width;		/* Average char width */
+  Num		ascent;			/* Height above baseline */
+  Num		descent;		/* Depth below baseline */
   BoolObj	fixed_width;		/* Is this a fixed width font? */
+  Num		ex;			/* Height of an x in this font */
+  Num		avg_char_width;		/* Average char width */
   WsRef		ws_ref;			/* Window system reference */
 End;
 
@@ -1053,10 +1048,6 @@ typedef struct
 #define GLOBAL extern
 #endif
 
-GLOBAL Image BLACK_IMAGE;
-GLOBAL Image WHITE_IMAGE;
-GLOBAL Image GREY50_IMAGE;
-
 GLOBAL Colour WHITE_COLOUR;
 GLOBAL Colour GREY25_COLOUR;
 GLOBAL Colour GREY50_COLOUR;
