@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2019-2022, University of Amsterdam
+    Copyright (c)  2019-2026, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -52,9 +52,13 @@ Event interface
 
 static void	free_event_callback(event_callback *cb);
 
-typedef int (*ev_func0)(void *);
-typedef int (*ev_func1)(void *, term_t);
-typedef int (*ev_func2)(void *, term_t, term_t);
+typedef bool (*ev_func0)(void *);
+typedef bool (*ev_func1)(void *, term_t);
+typedef bool (*ev_func2)(void *, term_t, term_t);
+
+typedef void (*ev_func0v)(void *);
+typedef void (*ev_func1v)(void *, term_t);
+typedef void (*ev_func2v)(void *, term_t, term_t);
 
 #ifdef O_PLMT
 #define INIT_LIST_LOCK(l) recursiveMutexInit(&(l)->lock)
@@ -109,8 +113,10 @@ link_event(event_list *list, event_callback *cb, bool last)
 }
 
 
-#define get_callback(closure, m, cb) LDFUNC(get_callback, closure, m, cb)
-static int
+#define get_callback(closure, m, cb) \
+	LDFUNC(get_callback, closure, m, cb)
+
+static bool
 get_callback(DECL_LD term_t closure, Module *m, term_t cb)
 { if ( !PL_strip_module(closure, m, cb) )
     return false;
@@ -197,9 +203,11 @@ register_event_hook(event_list **list, atom_t name,
 }
 
 
-#define get_event_listp(type, listpp, argc) LDFUNC(get_event_listp, type, listpp, argc)
-static int
-get_event_listp(DECL_LD term_t type, event_list ***listpp, size_t *argc)
+#define get_event_listp(type, listpp, argc) \
+	LDFUNC(get_event_listp, type, listpp, argc)
+
+static bool
+get_event_listp(DECL_LD term_t type, event_list ***listpp, int *argc)
 { atom_t name;
   size_t arity;
 
@@ -236,17 +244,20 @@ static const PL_option_t prolog_listen_options[] =
   { NULL_ATOM,		 0 }
 };
 
-#define prolog_listen(type, closure, options) LDFUNC(prolog_listen, type, closure, options)
-static int
+#define prolog_listen(type, closure, options) \
+	LDFUNC(prolog_listen, type, closure, options)
+
+static bool
 prolog_listen(DECL_LD term_t type, term_t closure, term_t options)
 { event_list **listp;
-  size_t argc;
+  int argc;
   atom_t as = ATOM_first;
   atom_t name = 0;
 
-  if ( options && !PL_scan_options(options, 0, /*OPT_ALL,*/
-				   "prolog_listen_option", prolog_listen_options,
-				   &as, &name) )
+  if ( options &&
+       !PL_scan_options(options, 0, /*OPT_ALL,*/
+			"prolog_listen_option", prolog_listen_options,
+			&as, &name) )
     return false;
 
   if ( !(as == ATOM_first || as == ATOM_last) )
@@ -279,7 +290,7 @@ static
 PRED_IMPL("prolog_unlisten", 2, prolog_unlisten, 0)
 { PRED_LD
   event_list **listp;
-  size_t argc;
+  int argc;
 
   if ( get_event_listp(A1, &listp, &argc) )
   { event_list *list;
@@ -342,11 +353,12 @@ PRED_IMPL("prolog_unlisten", 2, prolog_unlisten, 0)
 
 bool
 register_event_function(event_list **list, atom_t name,
-			bool last, int (*func)(),
-			void *closure, int argc)
+			bool last, void *func,
+			void *closure, int argc, unsigned int flags)
 { event_callback *cb = PL_malloc(sizeof(*cb));
   memset(cb, 0, sizeof(*cb));
   cb->argc = argc;
+  cb->flags = flags;
   cb->function = func;
   cb->closure.pointer = closure;
 
@@ -393,10 +405,12 @@ cleanupEvents(void)
 
 
 
-#define call_event_list(list, argc, argv) LDFUNC(call_event_list, list, argc, argv)
-static int
+#define call_event_list(list, argc, argv) \
+	LDFUNC(call_event_list, list, argc, argv)
+
+static bool
 call_event_list(DECL_LD event_list *list, int argc, term_t argv)
-{ int rc = true;
+{ bool rc = true;
 
   if ( list )
   { event_callback *ev;
@@ -404,19 +418,39 @@ call_event_list(DECL_LD event_list *list, int argc, term_t argv)
     LOCK_LIST(list);
     for(ev = list->head; ev; ev = ev->next)
     { if ( ev->function )
-      { switch(argc)
-	{ case 0:
-	    rc = (*(ev_func0)ev->function)(ev->closure.pointer);
-	    break;
-	  case 1:
-	    rc = (*(ev_func1)ev->function)(ev->closure.pointer, argv+1);
-	    break;
-	  case 2:
-	    rc = (*(ev_func2)ev->function)(ev->closure.pointer, argv+1, argv+2);
-	    break;
-	  default:
+      { if ( ison(ev, EV_CALLBACK_VOID) )
+	{ assert(argc >= ev->argc);
+	  switch(ev->argc)
+	  { case 0:
+	      (*(ev_func0v)ev->function)(ev->closure.pointer);
+	      break;
+	    case 1:
+	      (*(ev_func1v)ev->function)(ev->closure.pointer, argv+1);
+	      break;
+	    case 2:
+	      (*(ev_func2v)ev->function)(ev->closure.pointer, argv+1, argv+2);
+	      break;
+	    default:
+	      rc = false;
+	      assert(0);
+	  }
+	  if ( PL_exception(0) )
 	    rc = false;
-	    assert(0);
+	} else
+	{ switch(argc)
+	  { case 0:
+	      rc = (*(ev_func0)ev->function)(ev->closure.pointer);
+	      break;
+	    case 1:
+	      rc = (*(ev_func1)ev->function)(ev->closure.pointer, argv+1);
+	      break;
+	    case 2:
+	      rc = (*(ev_func2)ev->function)(ev->closure.pointer, argv+1, argv+2);
+	      break;
+	    default:
+	      rc = false;
+	      assert(0);
+	  }
 	}
       } else if ( ev->closure.term )
       { rc = rc &&
@@ -463,7 +497,7 @@ typedef struct delayed_event
 } delayed_event;
 
 
-static int
+static bool
 delayEvent(pl_event_type ev, va_list args)
 { GET_LD
 
@@ -499,7 +533,7 @@ delayEvent(pl_event_type ev, va_list args)
 }
 
 
-int
+bool
 delayEvents(void)
 { GET_LD
 
@@ -522,14 +556,14 @@ delayEvents(void)
 */
 
 int
-sendDelayedEvents(int noerror)
+sendDelayedEvents(bool noerror)
 { GET_LD
   int sent = 0;
 
   if ( --LD->event.delay_nesting == 0 )
   { Buffer b = LD->event.buffered;
     delayed_event *dev = baseBuffer(b, delayed_event);
-    int count = entriesBuffer(b, delayed_event);
+    size_t count = entriesBuffer(b, delayed_event);
 
     LD->event.buffered = NULL;
 
@@ -568,13 +602,13 @@ sendDelayedEvents(int noerror)
 }
 
 
-int
+bool
 PL_call_event_hook(pl_event_type ev, ...)
 { event_list **listp = event_list_location(ev);
 
-  if ( *listp && GD->cleaning != CLN_DATA )
+  if ( *listp && GD->halt.cleaning != CLN_DATA )
   { va_list args;
-    int rc;
+    bool rc;
 
     va_start(args, ev);
     rc = PL_call_event_hook_va(ev, args);
@@ -683,11 +717,11 @@ out:
 }
 
 
-int
+bool
 predicate_update_event(DECL_LD Definition def, atom_t action, Clause cl,
 		       unsigned flags)
 { wakeup_state wstate;
-  int rc;
+  bool rc;
 
   if ( (rc=saveWakeup(&wstate, true)) )
   { term_t av;
@@ -708,10 +742,10 @@ predicate_update_event(DECL_LD Definition def, atom_t action, Clause cl,
   return rc;
 }
 
-int
+bool
 table_answer_event(DECL_LD Definition def, atom_t action, term_t answer)
 { wakeup_state wstate;
-  int rc;
+  bool rc;
 
   if ( (rc=saveWakeup(&wstate, true)) )
   { term_t av;
@@ -735,11 +769,11 @@ table_answer_event(DECL_LD Definition def, atom_t action, term_t answer)
 }
 
 
-int
+bool
 retractall_event(DECL_LD Definition def, term_t head, functor_t start)
 { wakeup_state wstate;
   term_t av;
-  int rc = true;
+  bool rc = true;
 
   if ( !saveWakeup(&wstate, true) )
     return false;
