@@ -1048,7 +1048,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 	long sig = strtol(optval, &e, 10);
 
 	if ( e > optval && *e == EOS && sig >= 0 && sig < 32 )
-	  GD->signals.sig_alert = sig;
+	  GD->signals.sig_alert = (int) sig;
 	else
 	  return -1;
 #endif
@@ -1918,7 +1918,7 @@ emergency:
     memset(&PL_local_data,  0, sizeof(PL_local_data));
   }
 
-#ifdef __SANITIZE_ADDRESS__
+#if defined(__SANITIZE_ADDRESS__) && defined(HAVE_SANITIZER_LSAN_INTERFACE_H)
   char *s;
 
   if ( (s=getenv("ASAN_OPTIONS")) && strstr(s,"detect_leaks=1") )
@@ -1927,7 +1927,7 @@ emergency:
     if ( __lsan_do_recoverable_leak_check() )
     { fprintf(stderr, "Leaks detected; sleeping 60 sec.  Attach using\n"
 	     "   gdb -p %d\n", getpid());
-      sleep(60);
+      Pause(60.0);
     } else
     { fprintf(stderr, "No leaks detected\n");
     }
