@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2010-2019, University of Amsterdam
+    Copyright (c)  2010-2025, University of Amsterdam
                               VU University Amsterdam
+                              SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -195,9 +196,9 @@ typedef long long ssize_t;
 #include <string.h>
 #include <signal.h>
 
-#ifndef TRUE
-#define TRUE 1
-#define FALSE 0
+#ifndef true
+#define true 1
+#define false 0
 #endif
 
 #ifndef O_BINARY
@@ -286,21 +287,21 @@ static char *plsysinit;			/* -F file */
 static char *ctmp;			/* base executable */
 static char *pltmp;			/* base saved state */
 static char *out;			/* final output */
-static int  opt_o=FALSE;		/* -o out given */
-static int  opt_E=FALSE;		/* -E given */
+static int  opt_o=false;		/* -o out given */
+static int  opt_E=false;		/* -E given */
 
-static int build_defaults = FALSE;	/* don't ask Prolog for parameters*/
+static int build_defaults = false;	/* don't ask Prolog for parameters*/
 
-static int nostate = TRUE;		/* do not make a state */
-static int nolink = FALSE;		/* do not link */
-static int nolibswipl = FALSE;		/* do not link with -lswipl */
-static int shared = FALSE;		/* -shared: make a shared-object/DLL */
+static int nostate = true;		/* do not make a state */
+static int nolink = false;		/* do not link */
+static int nolibswipl = false;		/* do not link with -lswipl */
+static int shared = false;		/* -shared: make a shared-object/DLL */
 static char *soext = SO_EXT;		/* extension of shared object */
-static int embed_shared = FALSE;	/* -dll/-embed-shared: embed Prolog */
+static int embed_shared = false;	/* -dll/-embed-shared: embed Prolog */
 					/* in a DLL/.so file */
-static int verbose = TRUE;		/* verbose operation */
-static int fake = FALSE;		/* don't really do anything */
-static int show_version = FALSE;	/* --version */
+static int verbose = true;		/* verbose operation */
+static int fake = false;		/* don't really do anything */
+static int show_version = false;	/* --version */
 
 static void	removeTempFiles();
 static void	parseOptions(int argc, char **argv);
@@ -456,9 +457,9 @@ concatArgList(arglist *to, const char *prefix, arglist *from)
 
     buf[0] = UNQUOTED;
     if ( strchr(from->list[n], ' ') )
-      sprintf(buf+1, "%s\"%s\"", prefix, from->list[n]);
+      snprintf(buf+1, sizeof buf-1, "%s\"%s\"", prefix, from->list[n]);
     else
-      sprintf(buf+1, "%s%s", prefix, from->list[n]);
+      snprintf(buf+1, sizeof buf-1, "%s%s", prefix, from->list[n]);
     appendArgList(to, buf);
   }
 }
@@ -654,20 +655,20 @@ dispatchFile(const char *name)
     for( ; d->extension; d++ )
     { if ( strfeq(d->extension, ext) )
       { if ( d->list == &plfiles || d->list == &qlfiles )
-	  nostate = FALSE;
+	  nostate = false;
 	appendArgList(d->list, name);
-	return TRUE;
+	return true;
       }
     }
     if ( soext && strfeq(soext, ext) )
     { if ( d->list == &plfiles || d->list == &qlfiles )
-	nostate = FALSE;
+	nostate = false;
       appendArgList(&libs, name);
-      return TRUE;
+      return true;
     }
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -762,7 +763,7 @@ parseOptions(int argc, char **argv)
     } else if ( streq(opt, "--version") )	/* --version */
     { appendArgList(&coptions, opt);
       appendArgList(&cppoptions, opt);
-      show_version = TRUE;
+      show_version = true;
     } else if ( streq(opt, "-f") )		/* -f */
     { fake++;
     } else if ( streq(opt, "-c") )		/* -c */
@@ -773,7 +774,7 @@ parseOptions(int argc, char **argv)
       appendArgList(&cppoptions, opt);
     } else if ( streq(opt, "-E") )		/* -E */
     { nolink++;
-      opt_E = TRUE;
+      opt_E = true;
       appendArgList(&coptions, opt);
       appendArgList(&cppoptions, opt);
     } else if ( streq(opt, "-g") )		/* -g */
@@ -807,16 +808,16 @@ parseOptions(int argc, char **argv)
     { appendArgList(&coptions, opt);
       appendArgList(&cppoptions, opt);
     } else if ( streq(opt, "-build-defaults") )	/* -build-defaults */
-    { build_defaults = TRUE;
+    { build_defaults = true;
     } else if ( streq(opt, "-nostate") )	/* -nostate */
-    { nostate = TRUE;
+    { nostate = true;
     } else if ( streq(opt, "-state") )		/* -state */
-    { nostate = FALSE;
+    { nostate = false;
     } else if ( streq(opt, "-nolibswipl") )	/* -nolibswipl */
-    { nolibswipl = TRUE;
+    { nolibswipl = true;
     } else if ( streq(opt, "-dll") ||		/* -dll */
 		streq(opt, "-embed-shared") )   /* -embed-shared */
-    { embed_shared = TRUE;
+    { embed_shared = true;
 #if defined(HOST_TOOLCHAIN_MSC)
       appendArgList(&ldoptions, "/DLL");
 #else
@@ -826,15 +827,15 @@ parseOptions(int argc, char **argv)
 #endif
 #endif
     } else if ( streq(opt, "-shared") )		/* -shared */
-    { shared = TRUE;
-      nostate = TRUE;
+    { shared = true;
+      nostate = true;
 #ifdef SO_pic
       appendArgList(&coptions, SO_pic);
       appendArgList(&cppoptions, SO_pic);
 #endif
     } else if ( streq(opt, "-SHARED") )		/* -SHARED */
-    { shared = TRUE;
-      nostate = TRUE;
+    { shared = true;
+      nostate = true;
 #ifdef SO_PIC
       appendArgList(&coptions, SO_PIC);
       appendArgList(&cppoptions, SO_PIC);
@@ -859,7 +860,7 @@ parseOptions(int argc, char **argv)
     } else if ( streq(opt, "-o") )		/* -o out */
     { if ( argc > 1 )
       { out = argv[1];
-	opt_o = TRUE;
+	opt_o = true;
 	argc--, argv++;
       } else
 	usage();
@@ -977,7 +978,7 @@ tmpPath(char **store, const char *base)
 { if ( !*store )
   { char tmp[PATH_MAX];
 
-    sprintf(tmp, "%s%d", base, (int)getpid());
+    snprintf(tmp, sizeof tmp, "%s%d", base, (int)getpid());
     *store = strdup(tmp);
   }
 }
@@ -1006,7 +1007,7 @@ setLibDir(const char *libdir)
 
 
 static void
-fillDefaultOptions()
+fillDefaultOptions(void)
 { char tmp[1024];
   char *defcxx = PROG_CXX;
 
@@ -1065,25 +1066,28 @@ fillDefaultOptions()
   if ( !pllibdir )
   {
 #ifdef __WINDOWS__
-    sprintf(tmp, "%s/lib", plbase);
+    snprintf(tmp, sizeof tmp, "%s/lib", plbase);
 #else
-    sprintf(tmp, "%s/lib/%s", plbase, plarch);
+    snprintf(tmp, sizeof tmp, "%s/lib/%s", plbase, plarch);
 #endif
     prependArgList(&libdirs, tmp);
   }
-  sprintf(tmp, "%s/include", plbase);
+  snprintf(tmp, sizeof tmp, "%s/include", plbase);
   prependArgList(&includedirs, tmp);
+  snprintf(tmp, sizeof tmp, "-D_SWIPL_HOME=%s", plbase);
+  ensureOption(&coptions, tmp);
+  ensureOption(&cppoptions, tmp);
 }
 
 		 /*******************************
 		 *	   PROLOG OPTIONS	*
 		 *******************************/
 static void
-getPrologOptions()
+getPrologOptions(void)
 { FILE *fd;
   char cmd[512];
 
-  sprintf(cmd, "%s --dump-runtime-variables", pl);
+  snprintf(cmd, sizeof cmd, "%s --dump-runtime-variables", pl);
   if ( verbose )
     printf("\teval `%s`\n", cmd);
 
@@ -1146,9 +1150,9 @@ getPrologOptions()
     pclose(fd);
 
 #if defined(__WINDOWS__) && defined(HOST_OS_WINDOWS)
-    sprintf(buf, "%s/bin/%s", plbase, PROG_PL);
+    snprintf(buf, sizeof buf, "%s/bin/%s", plbase, PROG_PL);
 #else
-    sprintf(buf, "%s/bin/%s/%s", plbase, plarch, PROG_PL);
+    snprintf(buf, sizeof buf, "%s/bin/%s/%s", plbase, plarch, PROG_PL);
 #endif
     defaultPath(&plexe, buf);
   } else
@@ -1165,14 +1169,14 @@ char *
 shell_quote(char *to, const char *arg)
 { static const char needq[] = "#!|<>*?$'\"";
   const char *s;
-  int needquote = FALSE;
+  int needquote = false;
 
   if ( arg[0] == UNQUOTED )
   { arg++;				/* skip the not-quote marker */
   } else
   { for(s=arg; *s; s++)
     { if ( strchr(needq, *s) )
-      { needquote = TRUE;
+      { needquote = true;
 	break;
       }
     }
@@ -1330,7 +1334,7 @@ linkBaseExecutable()
 
 #if defined(HOST_TOOLCHAIN_MSC)
 { char tmp[PATH_MAX];
-  sprintf(tmp, "/out:%s", cout);
+  snprintf(tmp, sizeof tmp, "/out:%s", cout);
   prependArgList(&ldoptions, tmp);
 }
   concatArgList(&ldoptions, "", &ofiles);	/* object files */
@@ -1386,7 +1390,7 @@ linkSharedObject()
 #if defined(HOST_TOOLCHAIN_MSC)
   prependArgList(&ldoptions, "/dll");
 { char tmp[PATH_MAX];
-  sprintf(tmp, "/out:%s", soout);
+  snprintf(tmp, sizeof tmp, "/out:%s", soout);
   prependArgList(&ldoptions, tmp);
 }
   concatArgList(&ldoptions, "", &ofiles);	/* object files */
@@ -1414,7 +1418,7 @@ linkSharedObject()
 #ifdef SO_FORMAT_LDFLAGS			/* must specify output too */
   { char tmp[PATH_MAX];
     tmp[0] = UNQUOTED;
-    sprintf(&tmp[1], SO_FORMAT_LDFLAGS);
+    snprintf(&tmp[1], sizeof tmp-1, SO_FORMAT_LDFLAGS);
     prependArgList(&ldoptions, tmp);
   }
 #else
@@ -1443,7 +1447,7 @@ linkSharedObject()
 
 static void
 quoted_name(const char *name, char *plname)
-{ int needquote = TRUE;
+{ int needquote = true;
 
   if ( islower(CTOI(name[0])) )
   { const char *s = name+1;
@@ -1451,7 +1455,7 @@ quoted_name(const char *name, char *plname)
     for( ; *s && (isalnum(CTOI(*s)) || *s == '_'); s++)
       ;
     if ( *s == '\0' )
-      needquote = FALSE;
+      needquote = false;
   }
 
   if ( !needquote )
@@ -1690,7 +1694,7 @@ main(int argc, char **argv)
 
   putenv("PLLD=true");			/* for subprograms */
 
-  verbose = FALSE;
+  verbose = false;
 
   if ( argc > 2 && streq(argv[0], "-pl") )
     special = 2;
@@ -1714,11 +1718,11 @@ main(int argc, char **argv)
   defaultProgram(&pl, PROG_PL);
 
   if ( build_defaults )
-  { nostate = TRUE;			/* not needed and Prolog won't run */
+  { nostate = true;			/* not needed and Prolog won't run */
     defaultProgram(&cc, C_CC);
-#ifdef PLBASE
+#if defined(PLBASE)
     defaultPath(&plbase, PLBASE);
-#else
+#elif defined(PLHOME)
     defaultPath(&plbase, PLHOME);
 #endif
     defaultPath(&plarch, PLARCH);
@@ -1768,5 +1772,3 @@ main(int argc, char **argv)
 
   return 0;
 }
-
-
